@@ -1,5 +1,6 @@
 import secrets
 
+
 def gen_key():
     return secrets.token_hex(32)
 
@@ -8,5 +9,5 @@ def _main():
     print(gen_key())
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     _main()
