@@ -3,6 +3,7 @@ import os
 import inspect
 import time
 import re
+from pathlib import Path
 from os import PathLike
 from typing import (
     Any,
@@ -22,6 +23,8 @@ from typing import (
 )
 from typing_extensions import TypeGuard, ParamSpec, Concatenate
 
+from ..raw_rule import IMemo
+
 from ..core.make import MakeSummary
 
 from ..utils.dict_view import DictView
@@ -40,6 +43,8 @@ from .core import (
     IAtom,
 )
 from .event_logger import INoArgFunc
+from .fake_path import FakePath
+from ..memo import Memo
 
 K = TypeVar("K", bound=str)
 P = ParamSpec("P")
@@ -124,7 +129,9 @@ def require_init(
     return _method
 
 
-class Rule(IRule, Generic[K]):
+class Rule(  # pyright: ignore [reportIncompatibleMethodOverride]
+    IRule, IAtom, FakePath, Generic[K]
+):
     _raw_rule_id: int
     _info: GroupTreeInfo
     _name: Tuple[str]
@@ -133,6 +140,7 @@ class Rule(IRule, Generic[K]):
     _file_keys_hint: Optional[List[K]]
     _file_keys: List[K]
     _parent: IGroup
+    _memo: Memo[object] | None
 
     def __init_partial__(
         self,
@@ -155,11 +163,12 @@ class Rule(IRule, Generic[K]):
         method: object,
         args: Tuple[object, ...],
         kwargs: Dict[str, object],
+        noskip: bool,
     ):
         if self.initialized:
             raise RuntimeError(f"Rule {self.name} is already initialized")
 
-        self._init_main(yfiles, method, args, kwargs)
+        self._init_main(yfiles, method, args, kwargs, noskip)
         self._info.rules_to_be_init.remove(self._name)
 
     def __init_at_once__(
@@ -171,13 +180,14 @@ class Rule(IRule, Generic[K]):
         method: object,
         args: Tuple[object, ...],
         kwargs: Dict[str, object],
+        noskip: bool,
     ) -> Rule[K]:
         self._info = group_tree_info
         self._name = name
         self._file_keys_hint = None
         self._parent = parent
         self._file_keys = []  # do not read before the rule is fully initialized
-        self._init_main(yfiles, method, args, kwargs)
+        self._init_main(yfiles, method, args, kwargs, noskip)
 
         return self
 
@@ -196,16 +206,32 @@ class Rule(IRule, Generic[K]):
             return self._files[key]
 
     @property
-    def parent(self) -> IGroup:
+    def parent(  # pyright: ignore [reportIncompatibleMethodOverride]
+        self,
+    ) -> IGroup:
+        """Parent group node of this rule."""
         return self._parent
 
-    def touch(
+    def touch(  # pyright: ignore [reportIncompatibleMethodOverride]
         self,
         file: bool = True,
         memo: bool = True,
         create: bool = True,
         t: Union[float, None] = None,
     ) -> None:
+        """
+        Touch (set mtime to now) the output files and force the memo to record
+        the current input state.
+
+        Args:
+            file (bool):
+                if False, the output files won't be touched. Defaults to True.
+            memo (bool): if False, the memo won't be modified. Defaults to True.
+            create (bool):
+                if True, missing files will be created.
+                Otherwise, only the existing files will be touched.
+                This option has no effect with ``file=False``.
+        """
         logwriter = self._info.logwriter
 
         if t is None:
@@ -223,9 +249,12 @@ class Rule(IRule, Generic[K]):
                 logwriter.info(f"touch {f}")
 
         if memo:
-            self._info.rule_store.rules[self.raw_rule_id].update_memo()
+            self._info.rule_store.rules[self.raw_rule_id].memo.update()
 
     def clean(self) -> None:
+        """
+        Delete all the existing files of this rule.
+        """
         logwriter = self._info.logwriter
 
         for f in self.files.values():
@@ -245,7 +274,11 @@ class Rule(IRule, Generic[K]):
         method: object,
         args: object,
         kwargs: object,
+        noskip: bool,
     ):
+        if len(yfiles) == 0:
+            raise ValueError("Rules must have at least one output file.")
+
         args_ = (args, kwargs)
 
         # Add path prefix
@@ -280,7 +313,10 @@ class Rule(IRule, Generic[K]):
         method_ = _NoArgFunc(method, method_args, method_kwargs)
 
         # Create memo
-        memo = self._info.memo_factory(args_)
+        if noskip:
+            memo = _UnequalMemo()
+        else:
+            memo = self._info.memo_factory(next(iter(yp2f.values())), args_)
 
         # Update the RuleStore (create and add a new raw Rule)
         raw_rule = self._info.rule_store.add(
@@ -293,12 +329,21 @@ class Rule(IRule, Generic[K]):
         self._xfiles = list(xp2f)
         self._file_keys = list(yfiles)
 
+        if isinstance(memo, Memo):
+            self._memo = memo
+
         for k, f in yfiles.items():
             if k.isidentifier() and not hasattr(self, k):
                 setattr(self, k, f)
 
     @overload
-    def init(self, method: Callable[P, None], /) -> Callable[P, Rule[K]]:
+    def init(
+        self,
+        method: Callable[P, None],
+        /,
+        *,
+        noskip: bool = False,
+    ) -> Callable[P, Rule[K]]:
         ...
 
     @overload
@@ -312,16 +357,189 @@ class Rule(IRule, Generic[K]):
         ],
         method: Callable[P, object],
         /,
+        *,
+        noskip: bool = False,
     ) -> Callable[P, Rule[K]]:
         ...
 
+    @overload
     def init(
-        self, output_files: object, method: object = None, /
-    ) -> Callable[..., Rule[K]]:
-        return self._init(output_files, method, IFile_fact=File)
+        self,
+        output_files: Optional[
+            Union[
+                Mapping[K, StrOrPath],
+                Sequence[Union[K, PathLike[K]]],
+                K,
+                PathLike[K],
+            ]
+        ] = None,
+        /,
+        *,
+        noskip: bool = False,
+    ) -> Callable[[_T_deco_f], _T_deco_f]:
+        ...
+
+    def init(
+        self,
+        output_files: object = None,
+        method: object = None,
+        /,
+        *,
+        noskip: bool = False,
+    ) -> Callable[..., object]:
+        """
+        Create a temporary function to complete initialization of this rule.
+
+        Note:
+           This method must be called only for *uninitialized rules* which
+           do not have the output files, method, and method's arguments
+           assigned to themselvs yet.
+
+           For example, you have to call this method for rules of
+           :class:`StaticGroupBase`-like groups while you must not for rules
+           owned by :class:`RulesGroup` groups.
+
+        Args:
+            output_files:
+                if not specified, this rule's name will be used.
+                The following three forms are accepted.
+
+                * **Dict** (``{"key1": file1, "key2": file2, ...}``):
+                  ``key1``, ``key2``, ... are the *file keys* and ``file1``,
+                  ``file2``, ... are the *file paths*.
+                * **List** (``[file1, file2, ...]``):
+                  equivalent to a dict-form of
+                  ``{str(file1): file1, str(file2): file2}``
+                * **Atom** (``file``):
+                  equivalent to a dict-form of ``{str(file): file}``
+
+                *File keys* must be str. *File paths* may be either str or
+                PathLike including :class:`File` and
+                :class:`VFile`.
+                If a given file path is neither :class:`File` or
+                :class:`VFile`, it will be converted to :class:`File` by
+                ``File(file_path)``.
+
+            method: function to create the output files
+
+        Returns:
+            **rule_initializer**, a temporary function whose signature is the
+            same as the given ``method``.
+            Calling it as ``rule_adder(*args, **kwargs)`` completes
+            initialization of this rule.
+
+            While executing this rule, ``method`` is called as
+            ``method(*args, **kwargs)``.
+
+
+        Hint:
+
+            **Name Reference in File Paths**
+
+            A file path in a dict-form ``output_files`` may contain text
+            symbols ``"<R>"`` and ``"<F>"``, which will be replaced with
+            the rule's name and the corresponding file key, respectively.
+
+            For example, ``output_files={ "foo": Path("<R>-<F>.txt") }``
+            for a rule named "myrule" is equivalent to
+            ``output_files={ "foo": Path("myrule-foo.txt") }``.
+
+            In list/atom form of ``output_files``, you may use ``<R>`` too
+            but ``<F>`` is not allowed because the file keys are derived
+            from the file paths.
+
+
+            **Path Prefixing**
+
+            If given as a relative path, file paths get transformed by adding
+            the parent group's *path prefix* to the head.
+
+            For example, if the parent group's path prefix is
+            ``"output_dir/"``, ``output_files`` of ``{ "a": File("a.txt") }``
+            will be transformed into ``{ "a": File("out/a.txt") }``.
+
+            You can suppress this conversion by passing the path as an absolute
+            path.
+
+
+            **Rule-initializer and Argument Substitution**
+
+            ``Rule.init(output_files, method)`` returns a temporary function,
+            **rule_initializer** and you must further call it with the
+            arguments to be eventually passed to ``method`` like::
+
+                g.rule.init(output_files, method)(SELF, foo="bar")
+
+
+        Examples:
+            Basic usage:
+
+            .. testcode::
+
+                from __future__ import annotations
+                from pathlib import Path
+                from typing import Literal
+                from jtcmake import StaticGroupBase, Rule, SELF
+
+                def split_write(text: str, file1: Path, file2: Path):
+                    # Write first half of ``text`` to file1 and the rest to file2
+                    n = len(text)
+                    file1.write_text(text[: n // 2])
+                    file2.write_text(text[n // 2: n])
+
+
+                def cat(dst: Path, *srcs: Path):
+                    with dst.open("w") as f:
+                        f.writelines(src.read_text() for src in srcs)
+
+
+                class MyGroup(StaticGroupBase):
+                    __globals__ = globals()  # Only for Sphinx's doctest. Not necessary in normal situations.
+                    foo: Rule[Literal["a", "b"]]
+                    bar: Rule[str]
+                    buz: Rule[str]
+
+                    def init(self) -> MyGroup:
+                        '''
+                        Supplying keys other than "a" and "b" would be marked as
+                        a type error by type checkers such as Mypy and Pyright.
+                        '''
+                        self.foo.init({"a": "a.txt", "b": "b.txt"}, split_write)(
+                            "abcd", SELF[0], SELF[1]
+                        )
+
+                        '''
+                        The list below will be translated into `{"x.txt": "x.txt", "y.txt": "y.txt"}`
+                        (and then `{"x.txt": File("out/x.txt"), "y.txt": File("out/y.txt"}`)
+                        '''
+                        self.bar.init(["x.txt", "y.txt"], split_write)(
+                            "efgh", file1=SELF[0], file2=SELF[1]  # you can use keyword args
+                        )
+
+                        self.buz.init("w.txt", cat)(
+                            SELF, self.foo[0], self.foo[1], self.bar[0], self.bar[1]
+                        )
+
+                        return self
+
+                g = MyGroup("out").init()
+
+                g.make()
+
+                assert Path("out/a.txt").read_text() == "ab"
+                assert Path("out/b.txt").read_text() == "cd"
+                assert Path("out/x.txt").read_text() == "ef"
+                assert Path("out/y.txt").read_text() == "gh"
+                assert Path("out/w.txt").read_text() == "abcdefgh"
+
+                import shutil; shutil.rmtree("out")  # Cleanup for Sphinx's doctest
+        """
+        return self._init(output_files, method, File, noskip)
 
     @overload
-    def initvf(self, method: Callable[P, None], /) -> Callable[P, Rule[K]]:
+    def initvf(
+        self, method: Callable[P, None], /, *, noskip: bool = False
+    ) -> Callable[P, Rule[K]]:
         ...
 
     @overload
@@ -335,80 +553,74 @@ class Rule(IRule, Generic[K]):
         ],
         method: Callable[P, object],
         /,
+        *,
+        noskip: bool = False,
     ) -> Callable[P, Rule[K]]:
         ...
 
+    @overload
     def initvf(
-        self, output_files: object, method: object = None, /
-    ) -> Callable[..., Rule[K]]:
-        return self._init(output_files, method, IFile_fact=VFile)
+        self,
+        output_files: Optional[
+            Mapping[K, StrOrPath] | Sequence[K | PathLike[K]] | K | PathLike[K]
+        ] = None,
+        /,
+        *,
+        noskip: bool = False,
+    ) -> Callable[[_T_deco_f], _T_deco_f]:
+        ...
+
+    def initvf(
+        self,
+        output_files: object = None,
+        method: object = None,
+        /,
+        *,
+        noskip: bool = False,
+    ) -> Callable[..., object]:
+        """
+        Create a temporary function to initialize this rule.
+
+        This method is equal to :func:`init` except the default class
+        constructor is :class:`VFile` instead of :class:`File`.
+
+        Seealso:
+            :func:`init`
+        """
+        return self._init(output_files, method, VFile, noskip)
 
     def _init(
         self,
         output_files: object,
-        method: object = None,
-        /,
-        *,
+        method: object,
         IFile_fact: Callable[[StrOrPath], IFile],
-    ) -> Callable[P, Rule[K]]:
+        noskip: bool,
+    ) -> Callable[..., object]:
         if method is None:
-            output_files, method = self.name_tuple[-1], output_files
+            if callable(output_files):
+                output_files, method = self.name_tuple[-1], output_files
+            elif output_files is None:
+                output_files = self.name_tuple[-1]
 
         yfiles = parse_args_output_files(
             self.name_tuple[-1], self._file_keys_hint, output_files, IFile_fact
         )
 
-        def _init(*args: P.args, **kwargs: P.kwargs) -> Rule[K]:
-            self.__init_full__(yfiles, method, args, kwargs)
-            return self
+        if method is None:
 
-        return _init
+            def decorator(method: Callable[[], object]):
+                args, kwargs = Rule_init_parse_deco_func(method)
+                self.__init_full__(yfiles, method, args, kwargs, noskip)
+                return method
 
-    def init_deco(
-        self,
-        output_files: Optional[
-            Union[
-                Mapping[K, StrOrPath],
-                Sequence[Union[K, PathLike[K]]],
-                K,
-                PathLike[K],
-            ]
-        ] = None,
-    ) -> Callable[[_T_deco_f], _T_deco_f]:
-        return self._init_deco(output_files, IFile_fact=File)
+            return decorator
+        else:
 
-    def initvf_deco(
-        self,
-        output_files: Optional[
-            Union[
-                Mapping[K, StrOrPath],
-                Sequence[Union[K, PathLike[K]]],
-                K,
-                PathLike[K],
-            ]
-        ] = None,
-    ) -> Callable[[_T_deco_f], _T_deco_f]:
-        return self._init_deco(output_files, IFile_fact=VFile)
+            def rule_initializer(*args: P.args, **kwargs: P.kwargs) -> Rule[K]:
+                self.__init_full__(yfiles, method, args, kwargs, noskip)
+                return self
 
-    def _init_deco(
-        self,
-        output_files: object = None,
-        *,
-        IFile_fact: Callable[[StrOrPath], IFile],
-    ) -> Callable[[_T_deco_f], _T_deco_f]:
-        if output_files is None:
-            output_files = self.name_tuple[-1]
-
-        yfiles = parse_args_output_files(
-            self.name_tuple[-1], self._file_keys_hint, output_files, IFile_fact
-        )
-
-        def decorator(method: _T_deco_f):
-            args, kwargs = Rule_init_parse_deco_func(method)
-            self.__init_full__(yfiles, method, args, kwargs)
-            return method
-
-        return decorator
+            return rule_initializer
 
     def _get_info(self) -> GroupTreeInfo:
         return self._info
@@ -426,21 +638,25 @@ class Rule(IRule, Generic[K]):
         *,
         njobs: Optional[int] = None,
     ) -> MakeSummary:
-        """Make this rule and its dependencies
+        """
+        Make this rule and its dependencies
+
         Args:
             dry_run:
                 instead of actually excuting the methods,
                 print expected execution logs.
+
             keep_going:
                 If False (default), stop everything when a rule fails.
                 If True, when a rule fails, keep executing other rules
                 except the ones depend on the failed rule.
+
             njobs:
                 Maximum number of rules that can be made concurrently.
                 Defaults to 1 (single process, single thread).
 
-        See also:
-            See the description of jtcmake.make for more detail of njobs
+        Seealso:
+            :func:`jtcmake.make`
         """
         return make(
             self,
@@ -463,6 +679,29 @@ class Rule(IRule, Generic[K]):
     def xfiles(self) -> Collection[str]:
         return self._xfiles
 
+    @property
+    def memo_value(self) -> object:
+        return self[0].memo_value
+
+    @property
+    def real_value(self) -> object:
+        return self[0].real_value
+
+    @property
+    def memo_file(self) -> Path | None:
+        if self._memo is None:
+            return None
+        else:
+            return self._memo.memo_file
+
+
+class _UnequalMemo(IMemo):
+    def compare(self) -> bool:
+        return False
+
+    def update(self):
+        ...
+
 
 def Rule_init_parse_deco_func(
     method: object,
@@ -572,7 +811,7 @@ def parse_args_output_files(
     else:
         raise TypeError(
             "output_files must be str | PathLike | Sequence[str|PathLike] "
-            "| Mapping[str|PathLike]."
+            f"| Mapping[str|PathLike]. Given {output_files}"
         )
 
     if not _check_file_keys(outs, key_hints):
@@ -677,9 +916,16 @@ def _find_xfiles_in_args(
 
     def check(v: object):
         if isinstance(v, IFile):
-            absp = os.path.abspath(v)
+            f = v
+        elif isinstance(v, IRule):
+            f = next(iter(v.files.values()))
+        else:
+            f = None
+
+        if f is not None:
+            absp = os.path.abspath(f)
             if absp not in ypaths:
-                res[absp] = v
+                res[absp] = f
 
     map_structure(check, args)
 
